//=============================================================================
// Not a conventional projectile, this object serves as the end-point for an
// arc fired from the Electron Lance.
// http://mrevil.pwp.blueyonder.co.uk/unreal/
//=============================================================================
class SentinelProjectile_ElectronLance extends SentinelProjectile
	config(Mutator_SentinelGear);

var() const ParticleSystemComponent Arc;

var() float DamageInterval;

var SentinelWeapon_ElectronLance Lance;
var Sentinel Cannon;
var float Range;

simulated function PostBeginPlay()
{
	super.PostBeginPlay();

	SetArcParameters();
	Arc.ActivateSystem();

	if(Role == ROLE_Authority)
	{
		SetTimer(DamageInterval, true, 'Electrocute');
	}

	if(ProjEffects != none)
	{
		ProjEffects.SetActive(Base != none);
	}
}

simulated function OnSystemFinished(ParticleSystemComponent PSystem)
{
	ClearTimer('Electrocute');
	Sentinel(Instigator).WeaponComponent.DetachComponent(PSystem);
}

simulated function BaseChange()
{
	super.BaseChange();

	if(ProjEffects != none)
	{
		ProjEffects.SetActive(Base != none);
	}

	SetArcParameters();
}

simulated function SetArcParameters()
{
	if(WorldInfo.NetMode != NM_DedicatedServer)
	{
		if(Instigator != none)
		{
			Sentinel(Instigator).WeaponComponent.AttachComponentToSocket(Arc, Sentinel(Instigator).SWeapon.MuzzleSocketName);
		}

		Arc.SetActorParameter('BeamTarget', self);
		Arc.SetVectorParameter('BeamEndTangent', Vector(Rotation));
	}
}

function Electrocute()
{
	local Vector TraceStart, TraceEnd;
	local Actor HitActor;
	local Pawn HitPawn, BasePawn;
	local Vector HitLocation, HitNormal;

	TraceStart = Cannon.GetPawnViewLocation();
	TraceEnd = TraceStart + (Normal(Location - TraceStart) * Range);

	BasePawn = Pawn(Base);
	HitPawn = Lance.ClosestPawnToLineAngular(TraceStart, Normal(TraceEnd - TraceStart), FMin(VSize(TraceEnd - TraceStart) + 100.0, Lance.FireInfo.MaxRange), Lance.CosAttractionAngle);

	if(BasePawn != none)
	{
		if(HitPawn != none)
		{
			if(VSize(HitPawn.Location - Cannon.Location) < VSize(BasePawn.Location - Cannon.Location))
			{
				TraceEnd = TraceStart + (Normal(HitPawn.Location - TraceStart) * Range);
			}
			else
			{
				TraceEnd = TraceStart + (Normal(BasePawn.Location - TraceStart) * Range);
			}
		}
	}

	HitActor = Cannon.Trace(HitLocation, HitNormal, TraceEnd, TraceStart, true, vect(0.0, 0.0, 0.0),, TRACEFLAG_Bullet);

	if(HitActor == none)
	{
		HitLocation	= TraceEnd;
		HitNormal = Normal(TraceStart - TraceEnd);
	}
	else
	{
		if(UTWalkerBody(HitActor) != none)
		{
			HitPawn = UTWalkerBody(HitActor).WalkerVehicle;
			HitActor = HitPawn;
		}
		else
		{
			HitPawn = Pawn(HitActor);
		}

		if(HitPawn == none || !Cannon.IsSameTeam(HitPawn))
		{
			HitActor.TakeDamage(Damage, Cannon.InstigatorController, HitLocation, MomentumTransfer * Normal(TraceEnd - TraceStart), MyDamageType,, Cannon);
		}
	}

	SetLocation(HitLocation);
	SetRotation(Rotator(HitNormal));
	SetBase(HitActor);
}

//These projectiles have their location set directly, so they have no travel time.
static simulated function float StaticGetTimeToLocation(Vector TargetLoc, Vector StartLoc, Controller RequestedBy)
{
	return 0.0;
}

simulated function float GetTimeToLocation(Vector TargetLoc)
{
	return 0.0;
}

simulated function Destroyed()
{
	super.Destroyed();

	Arc.DeactivateSystem();
	Sentinel(Instigator).WeaponComponent.DetachComponent(Arc);
}

defaultproperties
{
	DamageInterval=0.2

	ProjFlightTemplate=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_ArcEnd'
	AmbientSound=SoundCue'SentinelGear_Resources.Sounds.Weapons.ElectronLance_ArcCue'
	MaxEffectDistance=20000.0
	ProjectileLightClass=class'ProjectileLight_ElectronLance'
	bProjectileLight=true

	Speed=0.0
	MaxSpeed=0.0
	AccelRate=0.0

	Damage=12
	DamageRadius=0.0
	MomentumTransfer=0.0
	NetCullDistanceSquared=144000000.0

	bCanBeDamaged=false
	bCollideActors=false
	bCollideWorld=false
	Physics=PHYS_None

	bNetTemporary=false
	bUpdateSimulatedPosition=true
	NetPriority=2.0
	NetUpdateFrequency=5.0
	bAlwaysRelevant=true //Hack. Ensures that arcs are always visible even if end-points are not.

	MyDamageType=class'UTDmgType_ElectronLance'
	LifeSpan=2.1

	Begin Object Class=ParticleSystemComponent Name=PSC0
		Template=ParticleSystem'SentinelGear_Resources.Effects.Weapons.P_ElectronLance_MainArc'
		Scale=0.25
		bAutoActivate=false
		OnSystemFinished=OnSystemFinished
	End Object
	Arc=PSC0
}